@@ -49,6 +49,7 @@ const VERSION: &'static str = env!("CARGO_PKG_VERSION");
 
 /// Winstons are a sub unit of the native Arweave network token, AR. There are 10<sup>12</sup> Winstons per AR.
 pub const WINSTONS_PER_AR: u64 = 1000000000000;
+pub const BLOCK_SIZE: u64 = 1024 * 256;
 
 #[derive(Serialize, Deserialize, Debug)]
 struct OraclePrice {
@@ -183,10 +184,14 @@ impl Arweave {
         Ok((winstons_per_bytes, usd_per_ar, usd_per_sol))
     }
 
-    pub async fn get_price_terms(&self) -> Result<(u64, u64), Error> {
-        let (prices1, prices2) = try_join(self.get_price(&1), self.get_price(&2)).await?;
-        let base = prices1.0.to_u64_digits()[0];
-        let incremental = prices2.0.to_u64_digits()[0] - &base;
+    pub async fn get_price_terms(&self, reward_mult: f32) -> Result<(u64, u64), Error> {
+        let (prices1, prices2) = try_join(
+            self.get_price(&(256 * 1024)),
+            self.get_price(&(256 * 1024 * 2)),
+        )
+        .await?;
+        let base = (prices1.0.to_u64_digits()[0] as f32 * reward_mult) as u64;
+        let incremental = (prices2.0.to_u64_digits()[0] as f32 * reward_mult) as u64 - &base;
         Ok((base, incremental))
     }
 
@@ -237,7 +242,8 @@ impl Arweave {
         };
 
         let data_len = data.len() as u64;
-        let reward = price_terms.0 + price_terms.1 * (data_len - 1);
+        let blocks_len = data_len / BLOCK_SIZE + (data_len % BLOCK_SIZE != 0) as u64;
+        let reward = price_terms.0 + price_terms.1 * (blocks_len - 1);
 
         Ok(Transaction {
             format: 2,
@@ -513,6 +519,7 @@ impl Arweave {
         from_keypair: &Keypair,
     ) -> Result<(Transaction, SigResponse), Error> {
         let lamports = std::cmp::max(&transaction.reward / RATE, FLOOR);
+
         let sol_tx = create_sol_transaction(solana_url, from_keypair, lamports).await?;
         let sig_response =
             get_sol_ar_signature(sol_ar_url, transaction.to_deep_hash_item()?, sol_tx).await?;
@@ -925,10 +932,10 @@ mod tests {
         // 2 items in the bundle
         assert_eq!(u64::from_le_bytes(bundle[0..8].try_into().unwrap()), 2);
 
-        // 2379 bytes in the first item
+        // 2892 bytes in the first item
         assert_eq!(u64::from_le_bytes(bundle[32..40].try_into().unwrap()), 2892);
 
-        // 2465 bytes in the secpnd item
+        // 2978 bytes in the secpnd item
         assert_eq!(
             u64::from_le_bytes(bundle[96..104].try_into().unwrap()),
             2978
@@ -959,7 +966,7 @@ mod tests {
             2u64
         );
 
-        // number of tag bytes is 51
+        // number of tag bytes is 52
         assert_eq!(
             u64::from_le_bytes(
                 bundle[(160 + 2 + 1024 + 1 + 1 + 8)..(160 + 2 + 1024 + 1 + 1 + 8 + 8)]
@@ -985,7 +992,8 @@ mod tests {
         assert_eq!(post_data_items.len(), 2);
         let mut pre_data_items_iter = pre_data_items.into_iter();
         let mut post_data_items_iter = post_data_items.into_iter();
-        // deep hash items and signatures are the same
+
+        // deep hash items and signatures are the same and singatures verify
         for _ in 0..2 {
             let pre_data_item = pre_data_items_iter.next().unwrap();
             let post_data_item = post_data_items_iter.next().unwrap();
@@ -1006,4 +1014,21 @@ mod tests {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_price_points() -> Result<(), Error> {
+        let mut price = 0 as u64;
+        println!("{:>6}  {:>12} {:>12}", "size", "winstons", "incremental");
+        println!("{:-<40}", "");
+        for p in 1..10 {
+            let size = p * 100 * 256;
+            let new_price = reqwest::get(format!("https://arweave.net/price/{}", size * 1024))
+                .await?
+                .json::<u64>()
+                .await?;
+            println!("{:>6}k {:>12} {:>12}", size, new_price, new_price - price);
+            price = new_price;
+        }
+        Ok(())
+    }
 }
